#include "liste.h"
/* Implémentation par liste à sentinelle
La liste contient toujours deux maillons, un
à la tête et un à la queue, appelés les
sentinelles. Ces maillons sont invisibles du
point de vue de l'utilisateur. */

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <stdbool.h>

typedef struct liste {
   struct maillon* tete;
   struct maillon* queue;
} liste_t;

typedef struct maillon {
   T val;
   struct maillon* suiv;
   struct maillon* prec;
} maillon_t;


liste_t* liste_vide(){
   liste_t* l = malloc(sizeof(liste_t));
   l->tete = malloc(sizeof(maillon_t));
   l->queue = malloc(sizeof(maillon_t));

   // TETE <-> QUEUE
   l->tete->suiv = l->queue;
   l->tete->prec = NULL;
   l->queue->prec = l->tete;
   l->queue->suiv = NULL;

   return l;
}


void print_liste(liste_t* l){
   // itérer sur les vrais maillons, i.e. en partant après la sentinelle de tête
   maillon_t* m = l->tete->suiv; 
   //jusqu'à atteindre la sentinelle de queue
   while (m->suiv != NULL){
      printf("%d", m->val);
      // mettre une double flèche s'il reste un vrai maillon à afficher
      if (m->suiv->suiv != NULL){
         printf(" <-> ");
      }
      m = m->suiv;
   }
   printf("\n");
}


void free_liste(liste_t* l){
   maillon_t* m = l->tete;
   while (m != NULL){
      maillon_t* prochain = m->suiv;
      free(m);
      m = prochain;
   }
   free(l);
}